#!/usr/bin/env python

from __future__ import print_function
import random

import my_add
import my_sub


# These can be changed, but make sure not to overflow `int`
a = random.randint(1, 1024)
b = random.randint(1, 1024)

if my_add.add(a, b) == a + b:
    print ("Test 1 Passed for SWIG custom source file extension")
else:
    print ("Test 1 FAILED for SWIG custom source file extension")
    exit(1)

if my_sub.sub(a, b) == a - b:
    print ("Test 2 Passed for SWIG custom source file extension")
else:
    print ("Test 2 FAILED for SWIG custom source file extension")
    exit(1)



from check_index import *

def check_reply(r):
    assert is_dict(r)
    assert sorted(r.keys()) == ["client-foo"]
    check_reply_client_foo(r["client-foo"])

def check_reply_client_foo(r):
    assert is_dict(r)
    assert sorted(r.keys()) == ["__test-v1", "__test-v2", "__test-v3", "unknown"]
    check_index__test(r["__test-v1"], 1, 3)
    check_index__test(r["__test-v2"], 2, 0)
    check_error(r["__test-v3"], "unknown query file")
    check_error(r["unknown"], "unknown query file")

def check_objects(o):
    assert is_list(o)
    assert len(o) == 2
    check_index__test(o[0], 1, 3)
    check_index__test(o[1], 2, 0)

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_cmake(index["cmake"])
check_reply(index["reply"])
check_objects(index["objects"])



from check_index import *

def check_objects(o):
    assert is_list(o)
    assert len(o) == 1
    check_index_object(o[0], "cache", 2, 0, check_object_cache)

def check_cache_entry(actual, expected):
    assert is_dict(actual)
    assert sorted(actual.keys()) == ["name", "properties", "type", "value"]

    assert is_string(actual["type"], expected["type"])
    assert is_string(actual["value"], expected["value"])

    def check_property(actual, expected):
        assert is_dict(actual)
        assert sorted(actual.keys()) == ["name", "value"]
        assert is_string(actual["value"], expected["value"])

    check_list_match(lambda a, e: is_string(a["name"], e["name"]), actual["properties"], expected["properties"], check=check_property)

def check_object_cache(o):
    assert sorted(o.keys()) == ["entries", "kind", "version"]
    # The "kind" and "version" members are handled by check_index_object.
    check_list_match(lambda a, e: is_string(a["name"], e["name"]), o["entries"], [
        {
            "name": "CM_OPTION_BOOL",
            "type": "BOOL",
            "value": "OFF",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing option()",
                },
            ],
        },
        {
            "name": "CM_SET_BOOL",
            "type": "BOOL",
            "value": "ON",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE BOOL)",
                },
                {
                    "name": "ADVANCED",
                    "value": "1",
                },
            ],
        },
        {
            "name": "CM_SET_FILEPATH",
            "type": "FILEPATH",
            "value": "dir1/dir2/empty.txt",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE FILEPATH)",
                },
            ],
        },
        {
            "name": "CM_SET_PATH",
            "type": "PATH",
            "value": "dir1/dir2",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE PATH)",
                },
                {
                    "name": "ADVANCED",
                    "value": "ON",
                },
            ],
        },
        {
            "name": "CM_SET_STRING",
            "type": "STRING",
            "value": "test",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE STRING)",
                },
            ],
        },
        {
            "name": "CM_SET_STRINGS",
            "type": "STRING",
            "value": "1",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE STRING) with STRINGS",
                },
                {
                    "name": "STRINGS",
                    "value": "1;2;3;4",
                },
            ],
        },
        {
            "name": "CM_SET_INTERNAL",
            "type": "INTERNAL",
            "value": "int2",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE INTERNAL)",
                },
            ],
        },
        {
            "name": "CM_SET_TYPE",
            "type": "STRING",
            "value": "1",
            "properties": [
                {
                    "name": "HELPSTRING",
                    "value": "Testing set(CACHE INTERNAL) with set_property(TYPE)",
                },
                {
                    "name": "ADVANCED",
                    "value": "0",
                },
            ],
        },
    ], check=check_cache_entry, allow_extra=True)

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_objects(index["objects"])



#!/usr/bin/env python3

import json
import os
import sys

if sys.version_info[0] >= 3:
    unicode = str

trace_file = None
expand = False

for i in sys.argv[1:]:
    if trace_file is None and not i.startswith('-'):
        trace_file = i
        continue

    if i in ['-e', '--expand']:
        expand = True

assert trace_file is not None
assert os.path.exists(trace_file)

if expand:
    msg_args = ['STATUS', 'fff', 'fff;sss;  SPACES !!!  ', ' 42  space in string!', '  SPACES !!!  ']
else:
    msg_args = ['STATUS', 'fff', '${ASDF}', ' ${FOO} ${BAR}', '  SPACES !!!  ']

required_traces = [
    {
        'args': ['STATUS', 'JSON-V1 str', 'spaces'],
        'cmd': 'message',
    },
    {
        'args': ['ASDF', 'fff', 'sss', '  SPACES !!!  '],
        'cmd': 'set',
    },
    {
        'args': ['FOO', '42'],
        'cmd': 'set',
    },
    {
        'args': ['BAR', ' space in string!'],
        'cmd': 'set',
    },
    {
        'args': msg_args,
        'cmd': 'message',
        'frame': 3 if expand else 2
    },
]

with open(trace_file, 'r') as fp:
    # Check for version (must be the first document)
    vers = json.loads(fp.readline())
    assert sorted(vers.keys()) == ['version']
    assert sorted(vers['version'].keys()) == ['major', 'minor']
    assert vers['version']['major'] == 1
    assert vers['version']['minor'] == 1

    for i in fp.readlines():
        line = json.loads(i)
        assert sorted(line.keys()) == ['args', 'cmd', 'file', 'frame', 'line', 'time']
        assert isinstance(line['args'], list)
        assert isinstance(line['cmd'], unicode)
        assert isinstance(line['file'], unicode)
        assert isinstance(line['frame'], int)
        assert isinstance(line['line'], int)
        assert isinstance(line['time'], float)

        for j in required_traces:
            # Compare the subset of required keys with line
            if {k: line[k] for k in j} == j:
                required_traces.remove(j)

assert not required_traces



from show_only_json_check import *

def check_kind(k):
    assert is_string(k)
    assert k == "ctestInfo"

def check_version(v):
    assert is_dict(v)
    assert sorted(v.keys()) == ["major", "minor"]
    assert is_int(v["major"])
    assert is_int(v["minor"])
    assert v["major"] == 1
    assert v["minor"] == 0

def check_backtracegraph(b):
    assert is_dict(b)
    assert sorted(b.keys()) == ["commands", "files", "nodes"]
    check_backtracegraph_commands(b["commands"])
    check_backtracegraph_files(b["files"])
    check_backtracegraph_nodes(b["nodes"])

def check_backtracegraph_commands(c):
    assert is_list(c)
    assert len(c) == 1
    assert is_string(c[0])
    assert c[0] == "add_test"

def check_backtracegraph_files(f):
    assert is_list(f)
    assert len(f) == 2
    assert is_string(f[0])
    assert is_string(f[1])
    assert f[0] == "file1"
    assert f[1] == "file0"

def check_backtracegraph_nodes(n):
    assert is_list(n)
    assert len(n) == 2
    node = n[0]
    assert is_dict(node)
    assert sorted(node.keys()) == ["file"]
    assert is_int(node["file"])
    assert node["file"] == 1
    node = n[1]
    assert is_dict(node)
    assert sorted(node.keys()) == ["command", "file", "line", "parent"]
    assert is_int(node["command"])
    assert is_int(node["file"])
    assert is_int(node["line"])
    assert is_int(node["parent"])
    assert node["command"] == 0
    assert node["file"] == 0
    assert node["line"] == 1
    assert node["parent"] == 0

def check_command(c):
    assert is_list(c)
    assert len(c) == 3
    assert is_string(c[0])
    check_re(c[0], "/cmake(\.exe)?$")
    assert is_string(c[1])
    assert c[1] == "-E"
    assert is_string(c[2])
    assert c[2] == "echo"

def check_reqfiles_property(p):
    assert is_dict(p)
    assert sorted(p.keys()) == ["name", "value"]
    assert is_string(p["name"])
    assert is_list(p["value"])
    assert p["name"] == "REQUIRED_FILES"
    assert len(p["value"]) == 1
    assert p["value"][0] == "RequiredFileDoesNotExist"

def check_willfail_property(p):
    assert is_dict(p)
    assert sorted(p.keys()) == ["name", "value"]
    assert is_string(p["name"])
    assert is_bool(p["value"])
    assert p["name"] == "WILL_FAIL"
    assert p["value"] == True

def check_resource_groups_property(p):
    assert is_dict(p)
    assert sorted(p.keys()) == ["name", "value"]
    assert is_string(p["name"])
    assert is_list(p["value"])
    assert p["name"] == "RESOURCE_GROUPS"
    assert len(p["value"]) == 3

    assert is_dict(p["value"][0])
    assert sorted(p["value"][0].keys()) == ["requirements"]
    assert is_list(p["value"][0]["requirements"])
    assert len(p["value"][0]["requirements"]) == 2
    assert is_dict(p["value"][0]["requirements"][0])
    assert sorted(p["value"][0]["requirements"][0].keys()) == \
        [".type", "slots"]
    assert is_string(p["value"][0]["requirements"][0][".type"])
    assert p["value"][0]["requirements"][0][".type"] == "threads"
    assert is_int(p["value"][0]["requirements"][0]["slots"])
    assert p["value"][0]["requirements"][0]["slots"] == 2
    assert is_string(p["value"][0]["requirements"][1][".type"])
    assert p["value"][0]["requirements"][1][".type"] == "gpus"
    assert is_int(p["value"][0]["requirements"][1]["slots"])
    assert p["value"][0]["requirements"][1]["slots"] == 4

    assert is_dict(p["value"][1])
    assert sorted(p["value"][1].keys()) == ["requirements"]
    assert is_list(p["value"][1]["requirements"])
    assert len(p["value"][1]["requirements"]) == 2
    assert is_dict(p["value"][1]["requirements"][0])
    assert sorted(p["value"][1]["requirements"][0].keys()) == \
        [".type", "slots"]
    assert is_string(p["value"][1]["requirements"][0][".type"])
    assert p["value"][1]["requirements"][0][".type"] == "threads"
    assert is_int(p["value"][1]["requirements"][0]["slots"])
    assert p["value"][1]["requirements"][0]["slots"] == 2
    assert is_string(p["value"][1]["requirements"][1][".type"])
    assert p["value"][1]["requirements"][1][".type"] == "gpus"
    assert is_int(p["value"][1]["requirements"][1]["slots"])
    assert p["value"][1]["requirements"][1]["slots"] == 4

    assert is_dict(p["value"][2])
    assert sorted(p["value"][2].keys()) == ["requirements"]
    assert is_list(p["value"][2]["requirements"])
    assert len(p["value"][2]["requirements"]) == 2
    assert is_dict(p["value"][2]["requirements"][0])
    assert sorted(p["value"][2]["requirements"][0].keys()) == \
        [".type", "slots"]
    assert is_string(p["value"][2]["requirements"][0][".type"])
    assert p["value"][2]["requirements"][0][".type"] == "gpus"
    assert is_int(p["value"][2]["requirements"][0]["slots"])
    assert p["value"][2]["requirements"][0]["slots"] == 2
    assert is_string(p["value"][2]["requirements"][1][".type"])
    assert p["value"][2]["requirements"][1][".type"] == "threads"
    assert is_int(p["value"][2]["requirements"][1]["slots"])
    assert p["value"][2]["requirements"][1]["slots"] == 4

def check_workingdir_property(p):
    assert is_dict(p)
    assert sorted(p.keys()) == ["name", "value"]
    assert is_string(p["name"])
    assert is_string(p["value"])
    assert p["name"] == "WORKING_DIRECTORY"
    assert p["value"].endswith("Tests/RunCMake/CTestCommandLine/ShowOnly")

def check_properties(p):
    assert is_list(p)
    assert len(p) == 4
    check_resource_groups_property(p[0])
    check_reqfiles_property(p[1])
    check_willfail_property(p[2])
    check_workingdir_property(p[3])

def check_tests(t):
    assert is_list(t)
    assert len(t) == 1
    test = t[0]
    assert is_dict(test)
    assert sorted(test.keys()) == ["backtrace", "command", "name", "properties"]
    assert is_int(test["backtrace"])
    assert test["backtrace"] == 1
    check_command(test["command"])
    assert is_string(test["name"])
    assert test["name"] == "ShowOnly"
    check_properties(test["properties"])

assert is_dict(ctest_json)
assert sorted(ctest_json.keys()) == ["backtraceGraph", "kind", "tests", "version"]
check_backtracegraph(ctest_json["backtraceGraph"])
check_kind(ctest_json["kind"])
check_version(ctest_json["version"])
check_tests(ctest_json["tests"])



from check_index import *
import os

class ExpectedVar(object):
    def __init__(self, name):
        self.name = name

class ExpectedList(object):
    def __init__(self, name):
        self.name = name

EXPECTED_TOOLCHAIN = {
    "language": "CXX",
    "compiler": {
        "path": ExpectedVar("CMAKE_CXX_COMPILER"),
        "id": ExpectedVar("CMAKE_CXX_COMPILER_ID"),
        "version": ExpectedVar("CMAKE_CXX_COMPILER_VERSION"),
        "target": ExpectedVar("CMAKE_CXX_COMPILER_TARGET"),
        "implicit": {
            "includeDirectories": \
                ExpectedList("CMAKE_CXX_IMPLICIT_INCLUDE_DIRECTORIES"),
            "linkDirectories": \
                ExpectedList("CMAKE_CXX_IMPLICIT_LINK_DIRECTORIES"),
            "linkFrameworkDirectories": \
                ExpectedList(
                    "CMAKE_CXX_IMPLICIT_LINK_FRAMEWORK_DIRECTORIES"),
            "linkLibraries": \
                ExpectedList("CMAKE_CXX_IMPLICIT_LINK_LIBRARIES"),
        }
    },
    "sourceFileExtensions": \
        ExpectedList("CMAKE_CXX_SOURCE_FILE_EXTENSIONS"),
}

def check_objects(o):
    assert is_list(o)
    assert len(o) == 1
    check_index_object(o[0], "toolchains", 1, 0, check_object_toolchains)

def check_object_toolchains(o):
    assert sorted(o.keys()) == ["kind", "toolchains", "version"]
    # The "kind" and "version" members are handled by check_index_object.
    toolchains = o["toolchains"]
    assert is_list(toolchains)

    # Other platform-specific toolchains may exist (like RC on Windows).
    has_cxx_toolchain = False
    for toolchain in toolchains:
        assert is_dict(toolchain)
        assert "language" in toolchain
        if toolchain["language"] == "CXX":
            check_object_toolchain(toolchain, EXPECTED_TOOLCHAIN)
            has_cxx_toolchain = True

    assert has_cxx_toolchain

def check_object_toolchain(o, expected):
    expected_keys = [
        key for (key, value) in expected.items()
        if is_string(value) or is_dict(value)
            or (type(value) in (ExpectedVar, ExpectedList)
                and variables[value.name]["defined"])]
    assert sorted(o.keys()) == sorted(expected_keys)

    for key in expected_keys:
        value = expected[key]
        if is_string(value):
            assert o[key] == value
        elif is_dict(value):
            check_object_toolchain(o[key], value)
        elif type(value) == ExpectedVar:
            assert o[key] == variables[value.name]["value"]
        elif type(value) == ExpectedList:
            expected_items = filter(
                None, variables[value.name]["value"].split(";"))
            check_list_match(lambda a, b: a == b, o[key], expected_items)
        else:
            assert False

with open(os.path.join(sys.argv[3], "toolchain_variables.json")) as f:
    variables = json.load(f)

assert is_dict(variables)
assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_objects(index["objects"])



from check_index import *

def check_reply(q):
    assert is_dict(q)
    assert sorted(q.keys()) == [
        "client-client-member",
        "client-empty-array",
        "client-empty-object",
        "client-json-bad-root",
        "client-json-empty",
        "client-json-extra",
        "client-not-file",
        "client-request-array-negative-major-version",
        "client-request-array-negative-minor-version",
        "client-request-array-negative-version",
        "client-request-array-no-major-version",
        "client-request-array-no-supported-version",
        "client-request-array-no-supported-version-among",
        "client-request-array-version-1",
        "client-request-array-version-1-1",
        "client-request-array-version-2",
        "client-request-negative-major-version",
        "client-request-negative-minor-version",
        "client-request-negative-version",
        "client-request-no-major-version",
        "client-request-no-version",
        "client-request-version-1",
        "client-request-version-1-1",
        "client-request-version-2",
        "client-requests-bad",
        "client-requests-empty",
        "client-requests-not-kinded",
        "client-requests-not-objects",
        "client-requests-unknown",
    ]
    expected = [
        (check_query_client_member, "client-client-member"),
        (check_query_empty_array, "client-empty-array"),
        (check_query_empty_object, "client-empty-object"),
        (check_query_json_bad_root, "client-json-bad-root"),
        (check_query_json_empty, "client-json-empty"),
        (check_query_json_extra, "client-json-extra"),
        (check_query_not_file, "client-not-file"),
        (check_query_requests_bad, "client-requests-bad"),
        (check_query_requests_empty, "client-requests-empty"),
        (check_query_requests_not_kinded, "client-requests-not-kinded"),
        (check_query_requests_not_objects, "client-requests-not-objects"),
        (check_query_requests_unknown, "client-requests-unknown"),
    ]
    for (f, k) in expected:
        assert is_dict(q[k])
        assert sorted(q[k].keys()) == ["query.json"]
        f(q[k]["query.json"])
    expected = [
        (check_query_response_array_negative_major_version,     "client-request-array-negative-major-version"),
        (check_query_response_array_negative_minor_version,     "client-request-array-negative-minor-version"),
        (check_query_response_array_negative_version,           "client-request-array-negative-version"),
        (check_query_response_array_no_major_version,           "client-request-array-no-major-version"),
        (check_query_response_array_no_supported_version,       "client-request-array-no-supported-version"),
        (check_query_response_array_no_supported_version_among, "client-request-array-no-supported-version-among"),
        (check_query_response_array_version_1,                  "client-request-array-version-1"),
        (check_query_response_array_version_1_1,                "client-request-array-version-1-1"),
        (check_query_response_array_version_2,                  "client-request-array-version-2"),
        (check_query_response_negative_major_version,           "client-request-negative-major-version"),
        (check_query_response_negative_minor_version,           "client-request-negative-minor-version"),
        (check_query_response_negative_version,                 "client-request-negative-version"),
        (check_query_response_no_major_version,                 "client-request-no-major-version"),
        (check_query_response_no_version,                       "client-request-no-version"),
        (check_query_response_version_1,                        "client-request-version-1"),
        (check_query_response_version_1_1,                      "client-request-version-1-1"),
        (check_query_response_version_2,                        "client-request-version-2"),
    ]
    for (f, k) in expected:
        assert is_dict(q[k])
        assert sorted(q[k].keys()) == ["query.json"]
        assert is_dict(q[k]["query.json"])
        assert sorted(q[k]["query.json"].keys()) == ["requests", "responses"]
        r = q[k]["query.json"]["requests"]
        assert is_list(r)
        assert len(r) == 1
        assert is_dict(r[0])
        assert r[0]["kind"] == "__test"
        r = q[k]["query.json"]["responses"]
        assert is_list(r)
        assert len(r) == 1
        assert is_dict(r[0])
        f(r[0])

def check_query_client_member(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["client", "responses"]
    assert is_dict(q["client"])
    assert sorted(q["client"].keys()) == []
    check_error(q["responses"], "'requests' member missing")

def check_query_empty_array(q):
    check_error(q, "query root is not an object")

def check_query_empty_object(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["responses"]
    check_error(q["responses"], "'requests' member missing")

def check_query_json_bad_root(q):
    check_error_re(q, "A valid JSON document must be either an array or an object value")

def check_query_json_empty(q):
    check_error_re(q, "value, object or array expected")

def check_query_json_extra(q):
    check_error_re(q, "Extra non-whitespace after JSON value")

def check_query_not_file(q):
    check_error_re(q, "failed to read from file")

def check_query_requests_bad(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["requests", "responses"]
    r = q["requests"]
    assert is_dict(r)
    assert sorted(r.keys()) == []
    check_error(q["responses"], "'requests' member is not an array")

def check_query_requests_empty(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["requests", "responses"]
    r = q["requests"]
    assert is_list(r)
    assert len(r) == 0
    r = q["responses"]
    assert is_list(r)
    assert len(r) == 0

def check_query_requests_not_kinded(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["requests", "responses"]
    r = q["requests"]
    assert is_list(r)
    assert len(r) == 4
    assert is_dict(r[0])
    assert sorted(r[0].keys()) == []
    assert is_dict(r[1])
    assert sorted(r[1].keys()) == ["kind"]
    assert is_dict(r[1]["kind"])
    assert is_dict(r[2])
    assert sorted(r[2].keys()) == ["kind"]
    assert is_list(r[2]["kind"])
    assert is_dict(r[3])
    assert sorted(r[3].keys()) == ["kind"]
    assert is_int(r[3]["kind"])
    r = q["responses"]
    assert is_list(r)
    assert len(r) == 4
    check_error(r[0], "'kind' member missing")
    check_error(r[1], "'kind' member is not a string")
    check_error(r[2], "'kind' member is not a string")
    check_error(r[3], "'kind' member is not a string")

def check_query_requests_not_objects(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["requests", "responses"]
    r = q["requests"]
    assert is_list(r)
    assert len(r) == 3
    assert is_int(r[0])
    assert is_string(r[1])
    assert is_list(r[2])
    r = q["responses"]
    assert is_list(r)
    assert len(r) == 3
    check_error(r[0], "request is not an object")
    check_error(r[1], "request is not an object")
    check_error(r[2], "request is not an object")

def check_query_requests_unknown(q):
    assert is_dict(q)
    assert sorted(q.keys()) == ["requests", "responses"]
    r = q["requests"]
    assert is_list(r)
    assert len(r) == 3
    assert is_dict(r[0])
    assert sorted(r[0].keys()) == ["kind"]
    assert r[0]["kind"] == "unknownC"
    assert is_dict(r[1])
    assert sorted(r[1].keys()) == ["kind"]
    assert r[1]["kind"] == "unknownB"
    assert is_dict(r[2])
    assert sorted(r[2].keys()) == ["kind"]
    assert r[2]["kind"] == "unknownA"
    r = q["responses"]
    assert is_list(r)
    assert len(r) == 3
    check_error(r[0], "unknown request kind 'unknownC'")
    check_error(r[1], "unknown request kind 'unknownB'")
    check_error(r[2], "unknown request kind 'unknownA'")

def check_query_response_array_negative_major_version(r):
    check_error(r, "'version' object 'major' member is not a non-negative integer")

def check_query_response_array_negative_minor_version(r):
    check_error(r, "'version' object 'minor' member is not a non-negative integer")

def check_query_response_array_negative_version(r):
    check_error(r, "'version' array entry is not a non-negative integer or object")

def check_query_response_array_no_major_version(r):
    check_error(r, "'version' object 'major' member missing")

def check_query_response_array_no_supported_version(r):
    check_error(r, "no supported version specified")

def check_query_response_array_no_supported_version_among(r):
    check_error(r, "no supported version specified among: 4.0 3.0")

def check_query_response_array_version_1(r):
    check_index__test(r, 1, 3)

def check_query_response_array_version_1_1(r):
    check_index__test(r, 1, 3) # always uses latest minor version

def check_query_response_array_version_2(r):
    check_index__test(r, 2, 0)

def check_query_response_negative_major_version(r):
    check_error(r, "'version' object 'major' member is not a non-negative integer")

def check_query_response_negative_minor_version(r):
    check_error(r, "'version' object 'minor' member is not a non-negative integer")

def check_query_response_negative_version(r):
    check_error(r, "'version' member is not a non-negative integer, object, or array")

def check_query_response_no_major_version(r):
    check_error(r, "'version' object 'major' member missing")

def check_query_response_no_version(r):
    check_error(r, "'version' member missing")

def check_query_response_version_1(r):
    check_index__test(r, 1, 3)

def check_query_response_version_1_1(r):
    check_index__test(r, 1, 3) # always uses latest minor version

def check_query_response_version_2(r):
    check_index__test(r, 2, 0)

def check_objects(o):
    assert is_list(o)
    assert len(o) == 2
    check_index__test(o[0], 1, 3)
    check_index__test(o[1], 2, 0)

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_cmake(index["cmake"])
check_reply(index["reply"])
check_objects(index["objects"])
